/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global window, tizen, console*/

/**
 * Application notification model module.
 * It is responsible for storing information about created notification.
 *
 * @module app.model.notification
 * @requires {@link app.common.strings}
 * @namespace app.model.notification
 * @memberof app.model
 */

// make sure that "app" namespace is created
window.app = window.app || {};

(function defineAppModelNotification(app) {
    'use strict';
    /*jshint maxstatements: 42*/

    /**
     * Strings model module reference.
     *
     * @memberof app.model.notification
     * @private
     * @type {object}
     */
    var commonStrings = app.common.strings,

        /**
         * Notification object.
         *
         * @memberof app.model.notification
         * @private
         * @type {NotificationManager}
         */
        notification = null,

        /**
         * Empty value constant.
         *
         * @memberof app.model.notification
         * @private
         * @const {string}
         */
        EMPTY_VALUE = '',

        /**
         * Default icon URL constant.
         *
         * @memberof app.model.notification
         * @private
         * @const {string}
         */
        DEFAULT_ICON_URL = 'images/main_icon.png',

        /**
         * Default subicon URL constant.
         *
         * @memberof app.model.notification
         * @private
         * @const {string}
         */
        DEFAULT_SUBICON_URL = 'images/sub_icon.png',

        /**
         * Default repeat constant.
         *
         * @memberof app.model.notification
         * @private
         * @const {number}
         */
        DEFAULT_REPEAT_VALUE = 1,

        /**
         * Max repeat constant.
         *
         * @memberof app.model.notification
         * @private
         * @const {number}
         */
        MAX_REPEAT_VALUE = 20,

        /**
         * Max LED on/off constant.
         *
         * @memberof app.model.notification
         * @private
         * @const {number}
         */
        MAX_LED_ON_OFF_VALUE = 10,

        /**
         * Default LED on/off constant.
         *
         * @memberof app.model.notification
         * @private
         * @const {number}
         */
        DEFAULT_LED_ON_OFF_VALUE = 0,

        /**
         * Default LED color constant.
         *
         * @memberof app.model.notification
         * @private
         * @const {string}
         */
        DEFAULT_LED_COLOR = '#178dc5',

        /**
         * Vibration on state constant.
         *
         * @memberof app.model.notification
         * @private
         * @const {boolean}
         */
        VIBRATION_ON_STATE = true,

        /**
         * Vibration off state constant.
         *
         * @memberof app.model.notification
         * @private
         * @const {boolean}
         */
        VIBRATION_OFF_STATE = false,

        /**
         * Progress percentage type constant.
         *
         * @memberof app.model.notification
         * @private
         * @const {string}
         */
        PROGRESS_PERCENTAGE_TYPE = 'PERCENTAGE',

        /**
         * Progress byte type constant.
         *
         * @memberof app.model.notification
         * @private
         * @const {string}
         */
        PROGRESS_BYTE_TYPE = 'BYTE',

        /**
         * Progress interval value in milliseconds.
         *
         * @memberof app.model.notification
         * @private
         * @const {number}
         */
        PROGRESS_INTERVAL = 10000,

        /**
         * Ongoing timeout value in milliseconds.
         *
         * @memberof app.model.notification
         * @private
         * @const {number}
         */
        ONGOING_TIMEOUT = 20000,

        /**
         * Notification type.
         *
         * @memberof app.model.notification
         * @private
         * @type {string}
         */
        type = EMPTY_VALUE,

        /**
         * Notification title.
         *
         * @memberof app.model.notification
         * @private
         * @type {string}
         */
        title = EMPTY_VALUE,

        /**
         * Notification title.
         *
         * @memberof app.model.notification
         * @private
         * @type {string}
         */
        content = EMPTY_VALUE,

        /**
         * Icon URL.
         *
         * @memberof app.model.notification
         * @private
         * @type {string}
         */
        iconURL = EMPTY_VALUE,

        /**
         * Subicon URL.
         *
         * @memberof app.model.notification
         * @private
         * @type {string}
         */
        subiconURL = EMPTY_VALUE,

        /**
         * Repeat value.
         *
         * @memberof app.model.notification
         * @private
         * @type {number}
         */
        repeat = DEFAULT_REPEAT_VALUE,

        /**
         * LED on value in pixels.
         *
         * @memberof app.model.notification
         * @private
         * @type {number}
         */
        ledOnPx = DEFAULT_LED_ON_OFF_VALUE,

        /**
         * LED on value in milliseconds.
         *
         * @memberof app.model.notification
         * @private
         * @type {number}
         */
        ledOnMs = DEFAULT_LED_ON_OFF_VALUE,

        /**
         * LED off value in pixels.
         *
         * @memberof app.model.notification
         * @private
         * @type {number}
         */
        ledOffPx = DEFAULT_LED_ON_OFF_VALUE,

        /**
         * LED off value in milliseconds.
         *
         * @memberof app.model.notification
         * @private
         * @type {number}
         */
        ledOffMs = DEFAULT_LED_ON_OFF_VALUE,

        /**
         * LED color.
         *
         * @memberof app.model.notification
         * @private
         * @type {string}
         */
        ledColor = DEFAULT_LED_COLOR,

        /**
         * Audio URL.
         *
         * @memberof app.model.notification
         * @private
         * @type {string}
         */
        audioURL = EMPTY_VALUE,

        /**
         * Vibration state.
         *
         * @memberof app.model.notification
         * @private
         * @type {boolean}
         */
        vibrationEnabled = VIBRATION_OFF_STATE,

        /**
         * Progress type.
         *
         * @memberof app.model.notification
         * @private
         * @type {string}
         */
        progressType = PROGRESS_PERCENTAGE_TYPE,

        /**
         * Main detail value.
         *
         * @memberof app.model.notification
         * @private
         * @type {string}
         */
        mainDetails = EMPTY_VALUE,

        /**
         * Sub detail value.
         *
         * @memberof app.model.notification
         * @private
         * @type {string}
         */
        subDetails = EMPTY_VALUE,

        /**
         * Action app ID.
         *
         * @memberof app.model.notification
         * @private
         * @type {string}
         */
        actionAppId = EMPTY_VALUE,

        /**
         * Progress interval counter.
         *
         * @memberof app.model.notification
         * @private
         * @type {number}
         */
        progressIntervalCounter = 0,

        /**
         * Object storing ids of progress intervals.
         *
         * @memberof app.model.notification
         * @private
         * @type {object}
         */
        progressIntervals = {};

    /**
     * Sets notification type.
     *
     * @memberof app.model.notification
     * @public
     * @param {string} value
     */
    function setType(value) {
        type = value.toUpperCase();
    }

    /**
     * Returns notification type.
     *
     * @memberOf app.model.notification
     * @public
     * @returns {string}
     */
    function getType() {
        return type;
    }

    /**
     * Sets notification title.
     *
     * @memberof app.model.notification
     * @public
     * @param {string} value
     */
    function setTitle(value) {
        title = value;
    }

    /**
     * Returns notification title.
     *
     * @memberof app.model.notification
     * @public
     * @returns {string}
     */
    function getTitle() {
        return title;
    }

    /**
     * Sets notification content.
     *
     * @memberof app.model.notification
     * @public
     * @param {string} value
     */
    function setContent(value) {
        content = value;
    }

    /**
     * Returns notification content.
     *
     * @memberof app.model.notification
     * @public
     * @returns {string}
     */
    function getContent() {
        return content;
    }

    /**
     * Sets URL of the notification icon.
     *
     * @memberof app.model.notification
     * @public
     * @param {string} value
     */
    function setIconURL(value) {
        iconURL = value;
    }

    /**
     * Returns URL of the notification icon.
     *
     * @memberof app.model.notification
     * @public
     * @returns {string}
     */
    function getIconURL() {
        if (iconURL === EMPTY_VALUE) {
            return DEFAULT_ICON_URL;
        } else {
            return iconURL;
        }
    }

    /**
     * Sets URL of the notification subicon.
     *
     * @memberof app.model.notification
     * @public
     * @param {string} value
     */
    function setSubiconURL(value) {
        subiconURL = value;
    }

    /**
     * Returns URL of the notification subicon.
     *
     * @memberof app.model.notification
     * @public
     * @returns {string}
     */
    function getSubiconURL() {
        if (subiconURL === EMPTY_VALUE) {
            return DEFAULT_SUBICON_URL;
        } else {
            return subiconURL;
        }
    }

    /**
     * Sets repeat parameter of the notification.
     *
     * @memberof app.model.notification
     * @public
     * @param {number} delta
     * @param {function} callback
     */
    function setRepeat(delta, callback) {
        repeat = Math.min(
            Math.max(repeat + delta, DEFAULT_REPEAT_VALUE),
            MAX_REPEAT_VALUE
        );

        callback(repeat);
    }

    /**
     * Returns repeat parameter of the notification.
     *
     * @memberof app.model.notification
     * @public
     * @returns {number}
     */
    function getRepeat() {
        return repeat;
    }

    /**
     * Returns max value of the LED on/off parameter of the notification.
     *
     * @memberof app.model.notification
     * @public
     * @returns {number}
     */
    function getLEDMax() {
        return MAX_LED_ON_OFF_VALUE;
    }

    /**
     * Sets LED on parameter of the notification.
     *
     * @memberof app.model.notification
     * @public
     * @param {number} valuePx
     * @param {number} valueMs
     */
    function setLEDOn(valuePx, valueMs) {
        ledOnPx = valuePx;
        ledOnMs = valueMs;
    }

    /**
     * Returns LED on parameter of the notification.
     *
     * @memberof app.model.notification
     * @public
     * @returns {number}
     */
    function getLEDOn() {
        return ledOnPx;
    }

    /**
     * Sets LED off parameter of the notification.
     *
     * @memberof app.model.notification
     * @public
     * @param {number} valuePx
     * @param {number} valueMs
     */
    function setLEDOff(valuePx, valueMs) {
        ledOffPx = valuePx;
        ledOffMs = valueMs;
    }

    /**
     * Returns LED off parameter of the notification.
     *
     * @memberof app.model.notification
     * @public
     * @returns {number}
     */
    function getLEDOff() {
        return ledOffPx;
    }

    /**
     * Sets LED color.
     *
     * @memberof app.model.notification
     * @public
     * @param {string} value
     */
    function setLEDColor(value) {
        ledColor = value;
    }

    /**
     * Returns LED color.
     *
     * @memberof app.model.notification
     * @public
     * @returns {string}
     */
    function getLEDColor() {
        return ledColor;
    }

    /**
     * Returns URL of the notification audio.
     *
     * @memberof app.model.notification
     * @public
     * @returns {string}
     */
    function getAudioURL() {
        return audioURL;
    }

    /**
     * Sets URL of the notification audio.
     *
     * @memberof app.model.notification
     * @public
     * @param {string} value
     */
    function setAudioURL(value) {
        audioURL = value;
    }

    /**
     * Returns vibration state of the notification.
     *
     * @memberof app.model.notification
     * @public
     * @returns {boolean}
     */
    function getVibrationState() {
        return vibrationEnabled;
    }

    /**
     * Turns the vibration state on.
     *
     * @memberof app.model.notification
     * @public
     */
    function turnVibrationOn() {
        vibrationEnabled = VIBRATION_ON_STATE;
    }

    /**
     * Turns the vibration state off.
     *
     * @memberof app.model.notification
     * @public
     */
    function turnVibrationOff() {
        vibrationEnabled = VIBRATION_OFF_STATE;
    }

    /**
     * Returns notification progress type.
     *
     * @memberof app.model.notification
     * @public
     * @returns {string}
     */
    function getProgressType() {
        return progressType;
    }

    /**
     * Sets percentage progress type.
     *
     * @memberof app.model.notification
     * @public
     */
    function setPercentageProgress() {
        progressType = PROGRESS_PERCENTAGE_TYPE;
    }

    /**
     * Sets byte progress type.
     *
     * @memberof app.model.notification
     * @public
     */
    function setByteProgress() {
        progressType = PROGRESS_BYTE_TYPE;
    }

    /**
     * Sets main details value.
     *
     * @memberof app.model.notification
     * @public
     * @param {string} value
     */
    function setMainDetails(value) {
        mainDetails = value;
    }

    /**
     * Returns main details value.
     *
     * @memberof app.model.notification
     * @public
     * @returns {string}
     */
    function getMainDetails() {
        return mainDetails;
    }

    /**
     * Sets sub details value.
     *
     * @memberof app.model.notification
     * @public
     * @param {string} value
     */
    function setSubDetails(value) {
        subDetails = value;
    }

    /**
     * Returns sub details value.
     *
     * @memberof app.model.notification
     * @public
     * @returns {string}
     */
    function getSubDetails() {
        return subDetails;
    }

    /**
     * Sets action app ID value.
     *
     * @memberof app.model.notification
     * @public
     * @param {string} value
     */
    function setActionAppId(value) {
        actionAppId = value;
    }

    /**
     * Returns action app ID value.
     *
     * @memberof app.model.notification
     * @public
     * @returns {string}
     */
    function getActionAppId() {
        return actionAppId;
    }

    /**
     * Clears notification data.
     *
     * @memberof app.model.notification
     * @public
     */
    function clearData() {
        type = EMPTY_VALUE;
        title = EMPTY_VALUE;
        content = EMPTY_VALUE;
        iconURL = EMPTY_VALUE;
        subiconURL = EMPTY_VALUE;
        repeat = DEFAULT_REPEAT_VALUE;
        ledOnPx = DEFAULT_LED_ON_OFF_VALUE;
        ledOnMs = DEFAULT_LED_ON_OFF_VALUE;
        ledOffPx = DEFAULT_LED_ON_OFF_VALUE;
        ledOffMs = DEFAULT_LED_ON_OFF_VALUE;
        ledColor = DEFAULT_LED_COLOR;
        audioURL = EMPTY_VALUE;
        vibrationEnabled = VIBRATION_OFF_STATE;
        progressType = PROGRESS_PERCENTAGE_TYPE;
        mainDetails = EMPTY_VALUE;
        subDetails = EMPTY_VALUE;
        actionAppId = EMPTY_VALUE;
    }

    /**
     * Updates notification progress.
     *
     * The purpose of this method is to show the user
     * how progress notification looks like in use.
     *
     * @memberof app.model.notification
     * @private
     * @param {StatusNotification} notificationStatus
     * @returns {number}
     */
    function updateProgress(notificationStatus) {
        var value = notificationStatus.progressValue;

        value += 0.2;
        notificationStatus.progressValue = value;
        notification.update(notificationStatus);

        return value;
    }

    /**
     * Removes ongoing notification.
     *
     * @memberof app.model.notification
     * @private
     * @param {StatusNotification} notificationStatus
     */
    function removeOngoing(notificationStatus) {
        notification.remove(notificationStatus.id);
    }

    /**
     * Prepares and returns notification data.
     *
     * @memberof app.model.notification
     * @public
     * @returns {object}
     */
    function getNotificationData() {
        /*jshint maxcomplexity:12 */

        var number = repeat > 1 ? repeat : null,
            subDetailsValue = commonStrings.nullIfEmptyString(subDetails),
            data = {};

        if (commonStrings.nullIfEmptyString(content)) {
            data.content = content;
        }

        if (commonStrings.nullIfEmptyString(iconURL)) {
            data.iconPath = iconURL;
        }

        if (commonStrings.nullIfEmptyString(subiconURL)) {
            data.subIconPath = subiconURL;
        }

        if (number) {
            data.number = number;
        }

        if (commonStrings.nullIfEmptyString(ledColor)) {
            data.ledColor = ledColor;
        }

        if (commonStrings.nullIfEmptyString(audioURL)) {
            data.soundPath = audioURL;
        }

        if (commonStrings.nullIfEmptyString(actionAppId)) {
            data.appId = actionAppId;
        }

        if (type === 'PROGRESS' &&
            commonStrings.nullIfEmptyString(progressType)) {
            data.progressType = progressType;
        }

        data.ledOnPeriod = ledOnMs;
        data.ledOffPeriod = ledOffMs;
        data.vibration = vibrationEnabled;

        if (commonStrings.nullIfEmptyString(mainDetails)) {
            data.detailInfo = [new tizen.NotificationDetailInfo(
                mainDetails,
                subDetailsValue
            )];

            data.detail = {mainDetails: mainDetails};

            if (subDetailsValue) {
                data.detail.subDetails += subDetailsValue;
            }
        }

        return data;
    }

    /**
     * Sends notification.
     *
     * @memberof app.model.notification
     * @public
     */
    function sendNotification() {
        var status = null,
            intervalId = 0;

        try {
            status = new tizen.StatusNotification(
                type,
                title,
                getNotificationData()
            );

            if (type === 'PROGRESS') {
                progressIntervalCounter += 1;
                intervalId = progressIntervalCounter;
                progressIntervals[progressIntervalCounter] = window.setInterval(
                    function onUpdateProgress() {
                        var value = updateProgress(status);

                        if (value > 0.9) {
                            notification.remove(status.id);
                            window.clearInterval(
                                progressIntervals[intervalId]
                            );
                            delete progressIntervals[intervalId];
                        }
                    },
                    PROGRESS_INTERVAL
                );
            } else if (type === 'ONGOING') {
                window.setTimeout(
                    function onProgressRemove() {
                        removeOngoing(status);
                    },
                    ONGOING_TIMEOUT
                );
            }

            notification.post(status);
        } catch (error) {
            console.warn('Send notification operation failed.', error);
        }
    }

    /**
     * Initializes the notification model module.
     *
     * @memberof app.model.notification
     * @public
     */
    function init() {
        notification = tizen.notification;
    }

    app.model = app.model || {};
    app.model.notification = {
        init: init,
        setType: setType,
        getType: getType,
        setTitle: setTitle,
        getTitle: getTitle,
        setContent: setContent,
        getContent: getContent,
        setIconURL: setIconURL,
        getIconURL: getIconURL,
        setSubiconURL: setSubiconURL,
        getSubiconURL: getSubiconURL,
        setRepeat: setRepeat,
        getRepeat: getRepeat,
        getLEDMax: getLEDMax,
        getLEDOn: getLEDOn,
        setLEDOn: setLEDOn,
        getLEDOff: getLEDOff,
        setLEDOff: setLEDOff,
        getLEDColor: getLEDColor,
        setLEDColor: setLEDColor,
        getAudioURL: getAudioURL,
        setAudioURL: setAudioURL,
        getVibrationState: getVibrationState,
        turnVibrationOn: turnVibrationOn,
        turnVibrationOff: turnVibrationOff,
        getProgressType: getProgressType,
        setPercentageProgress: setPercentageProgress,
        setByteProgress: setByteProgress,
        setMainDetails: setMainDetails,
        getMainDetails: getMainDetails,
        setSubDetails: setSubDetails,
        getSubDetails: getSubDetails,
        setActionAppId: setActionAppId,
        getActionAppId: getActionAppId,
        clearData: clearData,
        sendNotification: sendNotification,
        getNotificationData: getNotificationData
    };

})(window.app);
