/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global window, document*/

/**
 * Application UI code module.
 * It is responsible for managing user interface.
 *
 * @module app.ui.code
 * @requires {@link app.common.pages}
 * @requires {@link app.ui.templates}
 * @requires {@link app.model.notification}
 * @namespace app.ui.code
 * @memberof app.ui
 */

// make sure that "app" namespace is created
window.app = window.app || {};

(function defineAppUiCode(app) {
    'use strict';

    /**
     * Common pages module reference.
     *
     * @memberof app.ui.code
     * @private
     * @type {object}
     */
    var commonPages = app.common.pages,

        /**
         * Model notification module reference.
         *
         * @memberof app.ui.code
         * @private
         * @type {object}
         */
        modelNotification = app.model.notification,

        /**
         * UI templates module reference.
         *
         * @memberof app.ui.code
         * @private
         * @type {object}
         */
        uiTemplates = app.ui.templates,

        /**
         * Page element.
         *
         * @memberof app.ui.code
         * @private
         * @type {HTMLElement}
         */
        page = null,

        /**
         * Back button.
         *
         * @memberof app.ui.code
         * @private
         * @type {HTMLElement}
         */
        backBtn = null,

        /**
         * Text element.
         *
         * @memberof app.ui.code
         * @private
         * @type {HTMLElement}
         */
        text = null;

    /**
     * Updates the notification source code visible for user.
     *
     * @memberof app.ui.code
     * @private
     */
    function updateTextContent() {
        var textContent = '',
            data = modelNotification.getNotificationData(),
            dataExists = Object.keys(data).length > 0,
            attribute = null,
            attributeNameAndValue = null;

        textContent += uiTemplates.getCodeVariables();
        textContent += uiTemplates.getCodeTry();
        textContent += uiTemplates.getCodeStatusBegin();
        textContent += uiTemplates.getCodeNotificationType({
            type: modelNotification.getType()
        });
        textContent += uiTemplates.getCodeNotificationTitle({
            title: modelNotification.getTitle()
        });

        if (dataExists) {
            textContent += uiTemplates.getCodeDataBegin();
            for (attribute in data) {
                if (data.hasOwnProperty(attribute)) {
                    if (attribute === 'detailInfo') {
                        continue;
                    }

                    if (attribute === 'detail') {
                        textContent += uiTemplates.getCodeDetail({
                            mainDetails: data[attribute].mainDetails,
                            subDetails: data[attribute].subDetails
                        });
                        continue;
                    }

                    attributeNameAndValue = {
                        attribute: attribute,
                        value: data[attribute] + ''
                    };

                    if (typeof data[attribute] === 'string') {
                        textContent += uiTemplates
                            .getCodeAttributeString(attributeNameAndValue);
                    } else {
                        textContent += uiTemplates
                            .getCodeAttribute(attributeNameAndValue);
                    }
                }
            }
            textContent = textContent.slice(0, -1);
            textContent += uiTemplates.getCodeDataEnd();

        }

        textContent += uiTemplates.getCodeStatusEnd();
        textContent += uiTemplates.getCodeSend();
        textContent += uiTemplates.getCodeCatch();

        text.innerHTML = textContent;
    }

    /**
     * Clears the text content displaying notification source code.
     *
     * @memberof app.ui.code
     * @private
     */
    function clearTextContent() {
        text.innerHTML = '';
    }

    /**
     * Handles pageshow event.
     *
     * @memberof app.ui.code
     * @private
     */
    function onPageShow() {
        updateTextContent();
    }

    /**
     * Handles pagehide event.
     *
     * @memberof app.ui.code
     * @private
     */
    function onPageHide() {
        clearTextContent();
    }

    /**
     * Handles click event on back button.
     *
     * @memberof app.ui.code
     * @private
     */
    function onBackBtnClick() {
        commonPages.changePage('creator');
    }

    /**
     * Registers event listeners.
     *
     * @memberof app.ui.code
     * @private
     */
    function bindEvents() {
        page.addEventListener('pageshow', onPageShow);
        page.addEventListener('pagehide', onPageHide);
        backBtn.addEventListener('click', onBackBtnClick);
    }

    /**
     * Initializes the UI main module.
     *
     * @memberof app.ui.code
     * @public
     */
    function init() {
        page = document.getElementById('code');
        text = page.querySelector('.text');
        backBtn = page.querySelector('.button.back');
        bindEvents();
    }

    app.ui = app.ui || {};
    app.ui.code = {
        init: init
    };

})(window.app);
