/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global window, document*/

/**
 * Application UI creator option basic module.
 * It is responsible for managing UI content,
 * displaying notification basic option details.
 *
 * @module app.ui.creator.option.basic
 * @requires {@link app.model.notification}
 * @requires {@link app.ui.templates}
 * @namespace app.ui.creator.option.basic
 * @memberof app.ui.creator.option
 */

// make sure that "app" namespace is created
window.app = window.app || {};

(function defineAppUiCreatorOptionBasic(app) {
    'use strict';

    /**
     * UI templates module reference.
     *
     * @memberof app.ui.creator.option.basic
     * @private
     * @type {object}
     */
    var uiTemplates = app.ui.templates,

        /**
         * Model notification module reference.
         *
         * @memberof app.ui.creator.option.basic
         * @private
         * @type {object}
         */
        modelNotification = app.model.notification,

        /**
         * Enter key code.
         *
         * @memberof app.ui.creator.option.basic
         * @private
         * @type {number}
         */
        ENTER_KEYCODE = 13,

        /**
         * Incorrect CSS class.
         *
         * @memberof app.ui.creator.option.basic
         * @private
         * @const {string}
         */
        INCORRECT_CLASS = 'incorrect',

        /**
         * Parent container that contains option container.
         *
         * @memberof app.ui.creator.option.basic
         * @private
         * @type {HTMLElement}
         */
        detailsContainer = null,

        /**
         * Title input.
         *
         * @memberof app.ui.creator.option.basic
         * @private
         * @type {HTMLElement}
         */
        titleInput = null,

        /**
         * Content input.
         *
         * @memberof app.ui.creator.option.basic
         * @private
         * @type {HTMLElement}
         */
        contentInput = null;

    /**
     * Handles blur event on title text input.
     *
     * @memberof app.ui.creator.option.basic
     * @private
     */
    function onTitleInputBlur() {
        var trimmedValue = titleInput.value.trim();

        titleInput.value = trimmedValue;
        modelNotification.setTitle(trimmedValue);
    }

    /**
     * Handles focus event on title text input.
     *
     * @memberof app.ui.creator.option.basic
     * @private
     */
    function onTitleInputFocus() {
        titleInput.classList.remove(INCORRECT_CLASS);
    }

    /**
     * Handles blur event on content text input.
     *
     * @memberof app.ui.creator.option.basic
     * @private
     */
    function onContentInputBlur() {
        var trimmedValue = contentInput.value.trim();

        contentInput.value = trimmedValue;
        modelNotification.setContent(trimmedValue);
    }

    /**
     * Handles title.incorrect event.
     * Highlights text in title input in red.
     *
     * @memberof app.ui.creator.option.basic
     * @private
     */
    function onTitleIncorrect() {
        titleInput.classList.add(INCORRECT_CLASS);
    }

    /**
     * Handles keydown event on text input.
     *
     * @memberof app.ui.creator.option.basic
     * @private
     * @param {Event} ev
     */
    function onInputKeydown(ev) {
        if (ev.keyCode === ENTER_KEYCODE) {
            ev.target.blur();
        }
    }

    /**
     * Registers event listeners.
     *
     * @memberof app.ui.creator.option.basic
     * @private
     */
    function bindEvents() {
        titleInput.addEventListener('blur', onTitleInputBlur);
        titleInput.addEventListener('focus', onTitleInputFocus);
        titleInput.addEventListener('keydown', onInputKeydown);
        contentInput.addEventListener('blur', onContentInputBlur);
        contentInput.addEventListener('keydown', onInputKeydown);
        window.addEventListener('title.incorrect', onTitleIncorrect);
    }

    /**
     * Shows basic option content.
     *
     * @memberof app.ui.creator.option.basic
     * @public
     */
    function show() {
        detailsContainer.innerHTML = uiTemplates.getBasicOptionContent({
            title: modelNotification.getTitle(),
            content: modelNotification.getContent()
        });
        titleInput = detailsContainer.querySelector('.title-input');
        contentInput = detailsContainer.querySelector('.content-input');
        bindEvents();
    }

    /**
     * Initializes the UI creator option basic module.
     *
     * @memberof app.ui.creator.option.basic
     * @public
     */
    function init() {
        detailsContainer = document.querySelector('.details');
    }

    app.ui = app.ui || {};
    app.ui.creator = app.ui.creator || {};
    app.ui.creator.option = app.ui.creator.option || {};
    app.ui.creator.option.basic = {
        init: init,
        show: show
    };

})(window.app);
