/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global window, document*/

/**
 * Application UI creator option details module.
 * It is responsible for managing UI content,
 * displaying notification details option details.
 *
 * @module app.ui.creator.option.details
 * @requires {@link app.model.notification}
 * @requires {@link app.model.option}
 * @requires {@link app.ui.templates}
 * @namespace app.ui.creator.option.details
 * @memberof app.ui.creator.option
 */

// make sure that "app" namespace is created
window.app = window.app || {};

(function defineAppUiCreatorOptionDetails(app) {
    'use strict';

    /**
     * UI templates module reference.
     *
     * @memberof app.ui.creator.option.details
     * @private
     * @type {object}
     */
    var uiTemplates = app.ui.templates,

        /**
         * Model notification module reference.
         *
         * @memberof app.ui.creator.option.details
         * @private
         * @type {object}
         */
        modelNotification = app.model.notification,

        /**
         * Enter key code.
         *
         * @memberof app.ui.creator.option.details
         * @private
         * @type {number}
         */
        ENTER_KEYCODE = 13,

        /**
         * Parent container that contains option container.
         *
         * @memberof app.ui.creator.option.details
         * @private
         * @type {HTMLElement}
         */
        detailsContainer = null,

        /**
         * Main input.
         *
         * @memberof app.ui.creator.option.details
         * @private
         * @type {HTMLElement}
         */
        mainInput = null,

        /**
         * Sub input.
         *
         * @memberof app.ui.creator.option.details
         * @private
         * @type {HTMLElement}
         */
        subInput = null;

    /**
     * Handles blur event on main text input.
     *
     * @memberof app.ui.creator.option.details
     * @private
     */
    function onMainInputBlur() {
        var trimmedValue = mainInput.value.trim();

        mainInput.value = trimmedValue;
        modelNotification.setMainDetails(trimmedValue);
    }

    /**
     * Handles blur event on sub text input.
     *
     * @memberof app.ui.creator.option.details
     * @private
     */
    function onSubInputBlur() {
        var trimmedValue = subInput.value.trim();

        subInput.value = trimmedValue;
        modelNotification.setSubDetails(trimmedValue);
    }

    /**
     * Handles keydown event on text input.
     *
     * @memberof app.ui.creator.option.details
     * @private
     * @param {Event} ev
     */
    function onInputKeydown(ev) {
        if (ev.keyCode === ENTER_KEYCODE) {
            ev.target.blur();
        }
    }

    /**
     * Registers event listeners.
     *
     * @memberof app.ui.creator.option.details
     * @private
     */
    function bindEvents() {
        mainInput.addEventListener('blur', onMainInputBlur);
        mainInput.addEventListener('keydown', onInputKeydown);
        subInput.addEventListener('blur', onSubInputBlur);
        subInput.addEventListener('keydown', onInputKeydown);
    }

    /**
     * Shows details option content.
     *
     * @memberof app.ui.creator.option.details
     * @public
     */
    function show() {
        detailsContainer.innerHTML = uiTemplates.getDetailsOptionContent({
            mainText: modelNotification.getMainDetails(),
            subText: modelNotification.getSubDetails()
        });
        mainInput = detailsContainer.querySelector('.main-input');
        subInput = detailsContainer.querySelector('.sub-input');
        bindEvents();
    }

    /**
     * Initializes the UI creator option details module.
     *
     * @memberof app.ui.creator.option.details
     * @public
     */
    function init() {
        detailsContainer = document.querySelector('.details');
    }

    app.ui = app.ui || {};
    app.ui.creator = app.ui.creator || {};
    app.ui.creator.option = app.ui.creator.option || {};
    app.ui.creator.option.details = {
        init: init,
        show: show
    };

})(window.app);
