/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global window, document*/

/**
 * Application UI creator option icons module.
 * It is responsible for managing UI content,
 * displaying notification icons option details.
 *
 * @module app.ui.creator.option.icons
 * @requires {@link app.model.notification}
 * @requires {@link app.common.application}
 * @requires {@link app.model.option}
 * @requires {@link app.ui.templates}
 * @namespace app.ui.creator.option.icons
 * @memberof app.ui.creator.option
 */

// make sure that "app" namespace is created
window.app = window.app || {};

(function defineAppUiCreatorOptionIcons(app) {
    'use strict';

    /**
     * UI templates module reference.
     *
     * @memberof app.ui.creator.option.icons
     * @private
     * @type {object}
     */
    var uiTemplates = app.ui.templates,

        /**
         * Common application module reference.
         *
         * @memberof app.ui.creator.option.icons
         * @private
         * @type {object}
         */
        commonApplication = app.common.application,

        /**
         * Model notification module reference.
         *
         * @memberof app.ui.creator.option.icons
         * @private
         * @type {object}
         */
        modelNotification = app.model.notification,

        /**
         * Parent container that contains option container.
         *
         * @memberof app.ui.creator.option.icons
         * @private
         * @type {HTMLElement}
         */
        detailsContainer = null,

        /**
         * Main icon element.
         *
         * @memberof app.ui.creator.option.icons
         * @private
         * @type {HTMLElement}
         */
        mainIcon = null,

        /**
         * Sub icon element.
         *
         * @memberof app.ui.creator.option.icons
         * @private
         * @type {HTMLElement}
         */
        subIcon = null;

    /**
     * Handles icon image pick.
     *
     * Saves icon URL value to the model and updates its background.
     *
     * @memberof app.ui.creator.option.icons
     * @private
     * @param {string} url
     */
    function onMainIconPicked(url) {
        mainIcon.style.backgroundImage = 'url("' + url + '")';
        modelNotification.setIconURL(url);
    }

    /**
     * Handles subicon image pick.
     *
     * Saves subicon URL value to the model and updates its background.
     *
     * @memberof app.ui.creator.option.icons
     * @private
     * @param {string} url
     */
    function onSubIconPicked(url) {
        subIcon.style.backgroundImage = 'url("' + url + '")';
        modelNotification.setSubiconURL(url);
    }

    /**
     * Registers event listeners.
     *
     * @memberof app.ui.creator.option.icons
     * @private
     */
    function bindEvents() {
        mainIcon.addEventListener('click', onMainIconClick);
        subIcon.addEventListener('click', onSubIconClick);
    }

    /**
     * Shows icons option content.
     *
     * @memberof app.ui.creator.option.icons
     * @public
     */
    function show() {
        detailsContainer.innerHTML = uiTemplates.getIconsOptionContent({
            iconURL: modelNotification.getIconURL(),
            subiconURL: modelNotification.getSubiconURL()
        });
        mainIcon = detailsContainer.querySelector('.main-icon');
        subIcon = detailsContainer.querySelector('.sub-icon');
        bindEvents();
    }

    /**
     * Handles click event on main icon.
     *
     * @memberof app.ui.creator.option.icons
     * @private
     */
    function onMainIconClick() {
        commonApplication.pickImage(onMainIconPicked);
    }

    /**
     * Handles click event on sub icon.
     *
     * @memberof app.ui.creator.option.icons
     * @private
     * @param {Event} ev
     */
    function onSubIconClick(ev) {
        ev.stopPropagation();
        commonApplication.pickImage(onSubIconPicked);
    }

    /**
     * Initializes the UI creator option icons module.
     *
     * @memberof app.ui.creator.option.icons
     * @public
     */
    function init() {
        detailsContainer = document.querySelector('.details');
    }

    app.ui = app.ui || {};
    app.ui.creator = app.ui.creator || {};
    app.ui.creator.option = app.ui.creator.option || {};
    app.ui.creator.option.icons = {
        init: init,
        show: show
    };

})(window.app);
