/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global window, document*/

/**
 * Application UI creator option progress module.
 * It is responsible for managing UI content,
 * displaying notification progress option details.
 *
 * @module app.ui.creator.option.progress
 * @requires {@link app.model.notification}
 * @requires {@link app.model.option}
 * @requires {@link app.ui.templates}
 * @namespace app.ui.creator.option.progress
 * @memberof app.ui.creator.option
 */

// make sure that "app" namespace is created
window.app = window.app || {};

(function defineAppUiCreatorOptionProgress(app) {
    'use strict';

    /**
     * UI templates module reference.
     *
     * @memberof app.ui.creator.option.progress
     * @private
     * @type {object}
     */
    var uiTemplates = app.ui.templates,

        /**
         * Model notification module reference.
         *
         * @memberof app.ui.creator.option.progress
         * @private
         * @type {object}
         */
        modelNotification = app.model.notification,

        /**
         * Byte CSS class.
         *
         * @memberof app.ui.creator.option.progress
         * @private
         * @const {string}
         */
        BYTE_CLASS = 'byte',

        /**
         * Parent container that contains option container.
         *
         * @memberof app.ui.creator.option.progress
         * @private
         * @type {HTMLElement}
         */
        detailsContainer = null,

        /**
         * Container that contains vibration option details.
         *
         * @memberof app.ui.creator.option.progress
         * @private
         * @type {HTMLElement}
         */
        optionContainer = null,

        /**
         * Percentage button.
         *
         * @memberof app.ui.creator.option.progress
         * @private
         * @type {HTMLElement}
         */
        percentageBtn = null,

        /**
         * Byte button.
         *
         * @memberof app.ui.creator.option.progress
         * @private
         * @type {HTMLElement}
         */
        byteBtn = null;

    /**
     * Handles click event on percentage button.
     *
     * @memberof app.ui.creator.option.progress
     * @private
     */
    function onPercentageBtnClick() {
        optionContainer.classList.remove(BYTE_CLASS);
        modelNotification.setPercentageProgress();
    }

    /**
     * Handles click event on byte button.
     *
     * @memberof app.ui.creator.option.progress
     * @private
     */
    function onByteBtnClick() {
        optionContainer.classList.add(BYTE_CLASS);
        modelNotification.setByteProgress();
    }

    /**
     * Registers event listeners.
     *
     * @memberof app.ui.creator.option.progress
     * @private
     */
    function bindEvents() {
        percentageBtn.addEventListener('click', onPercentageBtnClick);
        byteBtn.addEventListener('click', onByteBtnClick);
    }

    /**
     * Shows progress option content.
     *
     * @memberof app.ui.creator.option.progress
     * @public
     */
    function show() {
        detailsContainer.innerHTML = uiTemplates.getProgressOptionContent({
            progressType: modelNotification.getProgressType()
        });
        optionContainer = detailsContainer.querySelector('.progress-option');
        percentageBtn = document.getElementById('percentage-btn');
        byteBtn = document.getElementById('byte-btn');
        bindEvents();
    }

    /**
     * Initializes the UI creator option progress module.
     *
     * @memberof app.ui.creator.option.progress
     * @public
     */
    function init() {
        detailsContainer = document.querySelector('.details');
    }

    app.ui = app.ui || {};
    app.ui.creator = app.ui.creator || {};
    app.ui.creator.option = app.ui.creator.option || {};
    app.ui.creator.option.progress = {
        init: init,
        show: show
    };

})(window.app);
