/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global window, document*/

/**
 * Application UI creator option sound module.
 * It is responsible for managing UI content,
 * displaying notification sound option details.
 *
 * @module app.ui.creator.option.sound
 * @requires {@link app.model.notification}
 * @requires {@link app.common.application}
 * @requires {@link app.model.option}
 * @requires {@link app.ui.templates}
 * @namespace app.ui.creator.option.sound
 * @memberof app.ui.creator.option
 */

// make sure that "app" namespace is created
window.app = window.app || {};

(function defineAppUiCreatorOptionSound(app) {
    'use strict';

    /**
     * UI templates module reference.
     *
     * @memberof app.ui.creator.option.sound
     * @private
     * @type {object}
     */
    var uiTemplates = app.ui.templates,

        /**
         * Common application module reference.
         *
         * @memberof app.ui.creator.option.sound
         * @private
         * @type {object}
         */
        commonApplication = app.common.application,

        /**
         * Model notification module reference.
         *
         * @memberof app.ui.creator.option.sound
         * @private
         * @type {object}
         */
        modelNotification = app.model.notification,

        /**
         * Parent container that contains option container.
         *
         * @memberof app.ui.creator.option.sound
         * @private
         * @type {HTMLElement}
         */
        detailsContainer = null,

        /**
         * Sound icon.
         *
         * @memberof app.ui.creator.option.sound
         * @private
         * @type {HTMLElement}
         */
        soundIcon = null,

        /**
         * Sound URL.
         *
         * @memberof app.ui.creator.option.sound
         * @private
         * @type {HTMLElement}
         */
        soundURL = null;

    /**
     * Handles audio pick.
     *
     * Saves audio URL value to the model and updates its title.
     *
     * @memberof app.ui.creator.option.sound
     * @private
     * @param {string} url
     */
    function onAudioPicked(url) {
        soundURL.innerText = url;
        modelNotification.setAudioURL(url);
    }

    /**
     * Handles click event on sound icon element.
     *
     * @memberof app.ui.creator.option.sound
     * @private
     */
    function onSoundIconClick() {
        commonApplication.pickAudio(onAudioPicked);
    }

    /**
     * Registers event listeners.
     *
     * @memberof app.ui.creator.option.sound
     * @private
     */
    function bindEvents() {
        soundIcon.addEventListener('click', onSoundIconClick);
    }

    /**
     * Shows sound option content.
     *
     * @memberof app.ui.creator.option.sound
     * @public
     */
    function show() {
        detailsContainer.innerHTML = uiTemplates.getSoundOptionContent({
            audioURL: modelNotification.getAudioURL()
        });
        soundIcon = document.getElementById('sound-icon');
        soundURL = detailsContainer.querySelector('.sound-url');
        bindEvents();
    }

    /**
     * Initializes the UI creator option sound module.
     *
     * @memberof app.ui.creator.option.sound
     * @public
     */
    function init() {
        detailsContainer = document.querySelector('.details');
    }

    app.ui = app.ui || {};
    app.ui.creator = app.ui.creator || {};
    app.ui.creator.option = app.ui.creator.option || {};
    app.ui.creator.option.sound = {
        init: init,
        show: show
    };

})(window.app);
