/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global window, document*/

/**
 * Application UI creator option vibration module.
 * It is responsible for managing UI content,
 * displaying notification vibration option details.
 *
 * @module app.ui.creator.option.vibration
 * @requires {@link app.model.notification}
 * @requires {@link app.model.option}
 * @requires {@link app.ui.templates}
 * @namespace app.ui.creator.option.vibration
 * @memberof app.ui.creator.option
 */

// make sure that "app" namespace is created
window.app = window.app || {};

(function defineAppUiCreatorOptionVibration(app) {
    'use strict';

    /**
     * UI templates module reference.
     *
     * @memberof app.ui.creator.option.vibration
     * @private
     * @type {object}
     */
    var uiTemplates = app.ui.templates,

        /**
         * Model notification module reference.
         *
         * @memberof app.ui.creator.option.vibration
         * @private
         * @type {object}
         */
        modelNotification = app.model.notification,

        /**
         * On CSS class.
         *
         * @memberof app.ui.creator.option.vibration
         * @private
         * @const {string}
         */
        ON_CLASS = 'on',

        /**
         * Parent container that contains option container.
         *
         * @memberof app.ui.creator.option.vibration
         * @private
         * @type {HTMLElement}
         */
        detailsContainer = null,

        /**
         * Container that contains vibration option details.
         *
         * @memberof app.ui.creator.option.vibration
         * @private
         * @type {HTMLElement}
         */
        optionContainer = null,

        /**
         * Vibration on button.
         *
         * @memberof app.ui.creator.option.vibration
         * @private
         * @type {HTMLElement}
         */
        vibrationOnBtn = null,

        /**
         * Vibration off button.
         *
         * @memberof app.ui.creator.option.vibration
         * @private
         * @type {HTMLElement}
         */
        vibrationOffBtn = null;

    /**
     * Handles click event on vibration on button.
     *
     * @memberof app.ui.creator.option.vibration
     * @private
     */
    function onVibrationOnBtnClick() {
        optionContainer.classList.add(ON_CLASS);
        modelNotification.turnVibrationOn();
    }

    /**
     * Handles click event on vibration off button.
     *
     * @memberof app.ui.creator.option.vibration
     * @private
     */
    function onVibrationOffBtnClick() {
        optionContainer.classList.remove(ON_CLASS);
        modelNotification.turnVibrationOff();
    }

    /**
     * Registers event listeners.
     *
     * @memberof app.ui.creator.option.vibration
     * @private
     */
    function bindEvents() {
        vibrationOnBtn.addEventListener('click', onVibrationOnBtnClick);
        vibrationOffBtn.addEventListener('click', onVibrationOffBtnClick);
    }

    /**
     * Shows vibration option content.
     *
     * @memberof app.ui.creator.option.vibration
     * @public
     */
    function show() {
        detailsContainer.innerHTML = uiTemplates.getVibrationOptionContent({
            vibrationEnabled: modelNotification.getVibrationState()
        });
        optionContainer = detailsContainer.querySelector('.vibration-option');
        vibrationOnBtn = document.getElementById('vibration-on-btn');
        vibrationOffBtn = document.getElementById('vibration-off-btn');
        bindEvents();
    }

    /**
     * Initializes the UI creator option vibration module.
     *
     * @memberof app.ui.creator.option.vibration
     * @public
     */
    function init() {
        detailsContainer = document.querySelector('.details');
    }

    app.ui = app.ui || {};
    app.ui.creator = app.ui.creator || {};
    app.ui.creator.option = app.ui.creator.option || {};
    app.ui.creator.option.vibration = {
        init: init,
        show: show
    };

})(window.app);
